@@ -1,5 +1,3 @@
-/* $Id: softhsm.cpp 7321 2013-09-23 12:05:15Z rb $ */
-
 /*
  * Copyright (c) 2008-2011 .SE (The Internet Infrastructure Foundation).
  * All rights reserved.
@@ -311,12 +309,11 @@ int main(int argc, char *argv[]) {
 #ifdef BOTAN_PRE_1_9_10_FIX
   Botan::Library_State* state = Botan::swap_global_state(0);
   Botan::swap_global_state(state);
-#else
-  Botan::Library_State* state = Botan::Global_State_Management::swap_global_state(0);
-  Botan::Global_State_Management::swap_global_state(state);
-#endif
 
   if(state) {
+#else
+  if(Botan::Global_State_Management::global_state_exists()) {
+#endif
     was_initialized = true;
   }
 
@@ -1053,7 +1050,7 @@ int removeSessionObjs(char *dbPath) {
 char* hexStrToBin(char *objectID, size_t idLength, size_t *newLen) {
   char *bytes = NULL;
 
-  if(idLength % 2 != 0) {
+  if(idLength < 2 || idLength % 2 != 0) {
     fprintf(stderr, "Error: Invalid length on hex string.\n");
     return NULL;
   }
