@@ -1,5 +1,3 @@
-/* $Id: SoftDatabase.cpp 7307 2013-09-12 13:30:52Z rb $ */
-
 /*
  * Copyright (c) 2008-2009 .SE (The Internet Infrastructure Foundation).
  * All rights reserved.
@@ -40,6 +38,12 @@
 #include <stdio.h>
 #include <string.h>
 #include <sched.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+#include <unistd.h>
+#include <errno.h>
+
 using std::string;
 
 // Rollback the object if it can't be saved
@@ -114,11 +118,22 @@ SoftDatabase::~SoftDatabase() {
 }
 
 CK_RV SoftDatabase::init(char *dbPath) {
+  // Create and set file permissions if the DB does not exist.
+  int fd = open(dbPath, O_CREAT, S_IRUSR | S_IWUSR);
+  if(fd == -1) {
+    char warnMsg[1024];
+    snprintf(warnMsg, sizeof(warnMsg), "Could not open the token database. errno=%i. "
+                                       "Probably wrong privileges: %s", errno, dbPath);
+    ERROR_MSG("init", warnMsg);
+    return CKR_TOKEN_NOT_PRESENT;
+  }
+  close(fd);
+
   // Open the database
   int result = sqlite3_open(dbPath, &db);
   if(result) {
     char warnMsg[1024];
-    snprintf(warnMsg, sizeof(warnMsg), "Could not open token database. Probably wrong privileges: %s", dbPath);
+    snprintf(warnMsg, sizeof(warnMsg), "Could not open the token database: %s", dbPath);
     ERROR_MSG("init", warnMsg);
     return CKR_TOKEN_NOT_PRESENT;
   }
